/**
 * FreeRDP: A Remote Desktop Protocol Implementation
 * Version includes
 *
 * Copyright 2013 Thincast Technologies GmbH
 * Copyright 2013 Bernhard Miklautz <bernhard.miklautz@thincast.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef WINPR_VERSION_H_
#define WINPR_VERSION_H_

#define WINPR_VERSION_MAJOR 3
#define WINPR_VERSION_MINOR 27
#define WINPR_VERSION_REVISION 0
#define WINPR_VERSION_SUFFIX ""
#define WINPR_API_VERSION "3"
#define WINPR_VERSION "3.27.0"
#define WINPR_VERSION_FULL "3.27.0"
#define WINPR_GIT_REVISION "3.27.0"

#endif // _WINPR_VERSION_H_
