#ifndef UWAC_CONFIG_H
#define UWAC_CONFIG_H

/* Include files */
/* #undef UWAC_HAVE_TM_GMTOFF */
/* #undef UWAC_HAVE_POLL_H */
/* #undef UWAC_HAVE_SYSLOG_H */
/* #undef UWAC_HAVE_JOURNALD_H */
/* #undef UWAC_HAVE_PIXMAN_REGION */
#define UWAC_HAVE_STRERROR_R

#endif /* UWAC_CONFIG_H */
